//ADAPTED
/*
    This file is part of Rekall.
    Copyright (C) 2013-2015

    Project Manager: Clarisse Bardiot
    Development & interactive design: Guillaume Jacquemin & Guillaume Marais (http://www.buzzinglight.com)

    This file was written by Guillaume Jacquemin.

    Rekall is a free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

function Tag(document) {
    NotificationEntityChanged.call(this, {});
    this.document = document;
    this.documentOriginal = this.document;
    this.version = document.currentVersion;
    this.versionOriginal = this.version;
    this.timeStart = 0;
    this.timeEnd = 1;
    this.color = "#FFFFFF";
    this.shown = false;
    this.thumbnail = undefined;
}

Tag.prototype = Object.create(NotificationEntityChanged.prototype);


Tag.keyToOpenAfterLoading = undefined;

Tag.prototype.getMetadata = function (metadataKey, metadataValue) {
    return this.document.getMetadata(metadataKey, this.version);
}
Tag.prototype.setMetadata = function (metadataKey, metadataValue) {
    let result = this.document.setMetadata(metadataKey, metadataValue, this.version);
    this.notifyChange();
    return result;
}
Tag.prototype.getMetadatas = function () {
    return this.document.getMetadatas(this.version);
}
Tag.prototype.isVideoOrAudio = function () {
    return this.document.isVideoOrAudio(this.version);
}
Tag.prototype.isAudio = function () {
    return this.document.isAudio(this.version);
}
Tag.prototype.isVideo = function () {
    return this.document.isVideo(this.version);
}
Tag.prototype.isMarker = function () {
    return this.document.isMarker(this.version);
}
Tag.prototype.isImage = function () {
    return this.document.isImage(this.version);
}
Tag.prototype.isLink = function () {
    return this.document.isLink(this.version);
}

Tag.prototype.getTagGradientColor = function() {
    var isOpera = !!window.opera || navigator.userAgent.indexOf(' OPR/') >= 0;  // Opera 8.0+ (UA detection to detect Blink/v8-powered Opera)
    var isFirefox = typeof InstallTrigger !== 'undefined';   // Firefox 1.0+
    var isSafari = Object.prototype.toString.call(window.HTMLElement).indexOf('Constructor') > 0;   // At least Safari 3+: "[object HTMLElementConstructor]"
    var isChrome = !!window.chrome && !isOpera;              // Chrome 1+
    var isIE = /*@cc_on!@*/false || !!document.documentMode; // At least IE6

    if (isOpera) {
        return "-o-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + this.color + " 100%)";
    }
    if (isFirefox) {
        return "-moz-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + this.color + " 100%)";
    }
    if ((isSafari) || (isChrome)) {
        return "-webkit-linear-gradient(right bottom,  rgba(20,46,51,1) 0%, " + this.color + " 100%)";
    }
}
Tag.prototype.getTimeStart = function () {
    return this.timeStart;
}
Tag.prototype.getTimeEnd = function () {
    return this.timeEnd;
}
Tag.prototype.setTimeStart = function (val) {
    if (!isNaN(val)) {
        this.timeStart = val;
        this.notifyChange();
    }
}
Tag.prototype.setTimeEnd = function (val) {
    if (!isNaN(val)) {
        this.timeEnd = val;
        this.notifyChange();
    }

}
Tag.prototype.isGoodVersion = function () {
    return this.version == this.document.goodVersion;
}
Tag.prototype.openFile = function () {
    return this.document.openFile();
}
Tag.prototype.openFinder = function () {
    return this.document.openFinder();
}
Tag.prototype.openQuickLook = function () {
    return this.document.openQuickLook();
}
Tag.prototype.downloadFile = function () {
    return this.document.downloadFile();
}
Tag.prototype.getDownloadLink = function (original = false) {
    return this.document.getDownloadLink(original);
}
Tag.prototype.getFileName = function () {
    return this.document.getFileName();
}
Tag.prototype.openBrowser = function () {
    return this.document.openBrowser();
}
Tag.prototype.openPopupEdit = function () {
    fillPopupEdit(this);
}
Tag.prototype.update = function (color, strong) {
    if (color != undefined) {
        this.colorRaw = color;
        this.color = color.toString();
        this.isMarkerCache = this.isMarker();
    }
    if (this.color == undefined)
        this.color = "#000000";
}
Tag.prototype.createTimelineDiv = function () {
    let that = this;
    let styleColor = "background-color: " + this.color + ";";
    let textColor2 = "color: rgba(255,255,255,.8)";

    let styleColor2 = styleColor;
    let styleImage = "";
    if (this.thumbnail.url != undefined) {
        styleImage = "background-image: url(" + encodeURI(this.thumbnail.url) + ");";
    } else styleImage = "background-color: rgba(255,255,255,.25)";

    let icnType = "";
    let tmpType = this.getMetadata("Rekall->Type");
    if (tmpType.indexOf("application/msword") >= 0) icnType = "background-image:url(css/images/icn-word.png);";
    else if (tmpType.indexOf("application/pdf") >= 0) icnType = "background-image:url(css/images/icn-pdf.png);";
    else if (tmpType.indexOf("application/") >= 0) icnType = "background-image:url(css/images/icn-document.png);";
    else if (tmpType.indexOf("audio/") >= 0) icnType = "background-image:url(css/images/icn-music.png);";
    else if (tmpType.indexOf("image/") >= 0) icnType = "background-image:url(css/images/icn-image.png);";
    else if (tmpType.indexOf("text/x-vcard") >= 0) icnType = "background-image:url(css/images/icn-user.png);";
    else if (tmpType.indexOf("text/") >= 0) icnType = "background-image:url(css/images/icn-document.png);";
    else if (tmpType.indexOf("video/") >= 0) icnType = "background-image:url(css/images/icn-video.png);";


    let html = "";
    html += "<div draggable=true class='flattentimeline_item' title='" + this.getMetadata("Rekall->Comments") + "' >";
    html += "<div class='flattentimeline_image'      style='" + styleImage + "'></div>";
    html += "<div class='flattentimeline_opacifiant' style='" + styleColor2 + "'></div>";
    html += "<div class='flattentimeline_type'		style='" + icnType + "' title='" + tmpType + "'></div>";
    html += "<div class='flattentimeline_title' 		style='" + textColor2 + "' title='" + this.getMetadata("Rekall->Name") + "'>" + this.getMetadata("Rekall->Name") + "</div>";
    html += "</div>";

    this.flattenTimelineDom = $(html);
    this.flattenTimelineDom.click(function () {
        that.openPopupEdit();
    });
    this.flattenTimelineDom.on({
        dragstart: function (event) {
            event.dataTransfer.setData("key", that.document.key);
            event.dataTransfer.setData("version", that.version);
        }
    });

    return this.flattenTimelineDom;

}
Tag.prototype.getLabels = function () {
    let labels = this.getMetadata('Rekall->Labels');
    if (!labels)
        return [];

    let isStringNullOrWhitspace = function (s) {
        if (typeof s !== "string" || !s) {
            return false;
        }

        return !s.trim();
    }

    return labels.split(';').filter(s => !isStringNullOrWhitspace(s));
}

function HighLightedTag(data) {
    Tag.call(this, data.Document);
}

HighLightedTag.prototype = Object.create(Tag.prototype);
HighLightedTag.prototype.createTimelineDiv = function () {
    let that = this;
    let styleColor = "background-color: " + this.color + ";";
    let textColor = "color: " + this.color + ";";

    let colorTransp = "";

    let styleColor2 = styleColor;
    let styleImage = "";
    if (this.thumbnail?.url) {
        styleImage = "background-image: -webkit-linear-gradient(right bottom,  rgba(20,46,51,.60) 0%,rgba(20,46,51,.90) 100%), url(" + this.thumbnail.url + "); background-image: -moz-linear-gradient(right bottom,  rgba(20,46,51,.60) 0%,rgba(20,46,51,.90) 100%), url(" + this.thumbnail.url + "); background-image: -o-linear-gradient(right bottom,  rgba(20,46,51,.60) 0%,rgba(20,46,51,.90) 100%), url(" + this.thumbnail.url + ");";
    } else {
        styleImage = "background-color: " + this.color + "; background-image: -webkit-linear-gradient(right bottom, rgba(20,46,51,.5) 0%,rgba(20,46,51,.8) 100%); background-image: -moz-linear-gradient(right bottom, rgba(20,46,51,.5) 0%,rgba(20,46,51,.8) 100%); background-image: -o-linear-gradient(right bottom, rgba(20,46,51,.5) 0%,rgba(20,46,51,.8) 100%);";
    }

    let tmpType = this.getMetadata("Rekall->Type");
    let typeTxt = tmpType.split("/")[1].replace(/marker/g, "note");

    let htmlHighlight = "";
    htmlHighlight += "<div draggable=true class='flattentimeline_item flattentimeline_highlightitem' style='" + colorTransp + " " + styleImage + "'>";
    htmlHighlight += "<div class='flattentimeline_title' 		style='" + textColor + "' title='" + this.getMetadata("Rekall->Name") + "'>" + this.getMetadata("Rekall->Name") + "</div>";


    if (this.getMetadata("Rekall->Comments") != "") {

        let tmpComments = this.getMetadata("Rekall->Comments");
        if (tmpComments.length > 150) {
            tmpComments = tmpComments.substring(0, 150) + "...";
        }

        let tmpcount = 0;
        let tmpIndex = tmpComments.indexOf("<br/>");
        while ((tmpcount < 3) && (tmpIndex != -1)) {
            tmpcount++;
            tmpIndex = tmpComments.indexOf("<br/>", tmpIndex + 1);
        }
        if (tmpIndex != -1) {
            tmpComments = tmpComments.substring(0, tmpIndex) + "...";
        }
        htmlHighlight += "<div class='flattentimeline_description'>" + tmpComments + "</div>";
    }

    if (this.getMetadata("Rekall->Author") != "") htmlHighlight += "<div class='flattentimeline_author'>" + this.getMetadata("Rekall->Author") + "</div>";


    htmlHighlight += "<div class='flattentimeline_typeTxt'		>" + typeTxt + "</div>";
    htmlHighlight += "<div class='flattentimeline_opacifiant' style='" + styleColor2 + "'></div>";
    htmlHighlight += "</div>";

    this.flattenTimelineDom = $(htmlHighlight);
    this.flattenTimelineDom.click(function () {
        that.openPopupEdit();
    });
    this.flattenTimelineDom.on({
        dragstart: function (event) {
            event.dataTransfer.setData("key", that.document.key);
            event.dataTransfer.setData("version", that.version);
        }
    });

    return this.flattenTimelineDom;
}


const TagFactory = function (document) {

    if (document.getMetadata("Rekall->Highlight", document.currentVersion)) {
        return new HighLightedTag({document: document})
    }

    return new Tag(document);
}